/*
 Przygotowanie bazy danych

 create table Osoba( ID int not null primary key, Imie varchar2(30), Nazwisko varchar2(50), Pensja int)

 begin
   insert into Osoba values( 1, 'Jan', 'Kowalski', 1000 );
   insert into Osoba values( 2, 'Ewa', 'Solska', 1200 );
   insert into Osoba values( 3, 'Boydar', 'Nowak', 1500 );
   insert into Osoba values( 4, 'Adam', 'Adamski', 1700 );
   insert into Osoba values( 5, 'Zygmunt', 'Riposta', 1240 );
   insert into Osoba values( 6, 'Alicja', 'Gleba', 1600 );
   insert into Osoba values( 7, 'Tania', 'Borwka', 1560 );
   insert into Osoba values( 8, 'Natasza', 'Lizacka', 2500 );
   insert into Osoba values( 9, 'Stan', 'Taczkowski', 1900 );
 end;

 select * from Osoba;
*/

import java.awt.*;
import java.awt.event.*;
import java.sql.*;
import java.util.*;
import javax.sql.rowset.CachedRowSet;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import oracle.jdbc.pool.*;
import com.sun.rowset.*;

public class TableBinding extends JFrame implements TableModelListener {
    static final long serialVersionUID = 49383948;
    DefaultTableModel tableModel;
    CachedRowSet rowset;
    
    private JPanel panel;
    private JButton btnSend;
    private JButton btnReceive;
    private JButton btnRestore;
    private JTable dataTable;
    
    private final static String TABLE = "Osoba";

    public TableBinding() {
        try {
            initComponents();
            rowset = new CachedRowSetImpl();
            rowset.setType(ResultSet.TYPE_SCROLL_INSENSITIVE);
            rowset.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        receive();
    }

    private void initComponents() {
        panel = new JPanel();
        btnSend = new JButton("Send");
        btnReceive = new JButton("Receive");
        btnRestore = new JButton("Restore");
        
        addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent evt) {
                    System.exit(0);
                }
            }
        );
        
        btnSend.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    send();
                }
            }
        );

        btnReceive.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    receive();
                }
            }
        );

        btnRestore.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    restore();
                }
            }
        );
        
        panel.add(btnSend);
        panel.add(btnReceive);
        panel.add(btnRestore);
        getContentPane().add(panel, BorderLayout.SOUTH);
        dataTable = new JTable();
        JScrollPane scrollPane = new JScrollPane(dataTable);
        getContentPane().add(scrollPane, BorderLayout.CENTER);
        pack();
        setSize(400, 200);
        setVisible(true);
    }
    
    public OracleDataSource getDataSource() throws ClassNotFoundException {
        OracleDataSource ds = null;
        try {
            ds = new OracleDataSource();
            ds.setURL("jdbc:oracle:thin:pawel/oracle10g@localhost:1521:XE");
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return ds;
    }
    
    public static void main(String[] args) {
        new TableBinding();
    }
    
    public void tableChanged(TableModelEvent event) {
        int row = event.getFirstRow();
        int column = event.getColumn();
        if (row == -1 || column == -1)
            return;
        Object data = tableModel.getValueAt(row, column);
        try {
            rowset.absolute(row+1);
            rowset.updateObject(column+1, data);
            rowset.updateRow();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println("XXXXXXXXXX");
    }    

    public void receive() {
        try {
            OracleDataSource ds = getDataSource();
            Connection conn = ds.getConnection();

            rowset.release();
            rowset.setCommand("SELECT Id, Imie, Nazwisko, Pensja FROM Osoba");
            rowset.execute(conn);
            conn.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        tableModel = rsToTable(rowset);
        dataTable.setModel(tableModel);
        dataTable.repaint();
    }
    
    public void send() {
        try {
            OracleDataSource ds = getDataSource();
            Connection conn = ds.getConnection();
            rowset.setTableName(TABLE);
            rowset.acceptChanges(conn);
            conn.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public void restore() {
        try {
            rowset.restoreOriginal();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        tableModel = rsToTable(rowset);
        dataTable.setModel(tableModel);
        dataTable.repaint();
    }
    
    public DefaultTableModel rsToTable(CachedRowSet rs) {
        try {
            int nColumns = rs.getMetaData().getColumnCount();
            Vector<String> cNames = new Vector<String>(nColumns);
            for (int i=1; i<=nColumns; i++)
                cNames.add(rs.getMetaData().getColumnLabel(i));
            DefaultTableModel tm = new DefaultTableModel();
            tm.addTableModelListener(this);
            Vector<Vector<String>> data = new Vector<Vector<String>>();
            Vector<String> row = new Vector<String>();
            rs.beforeFirst();
            while (rs.next()) {
                row = new Vector<String>();
                for (int i=1; i<=nColumns; i++)
                    row.add(rs.getString(i));
                data.add(row);
            }
            tm.setDataVector(data, cNames);
            return tm;
        }
        catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
}
